#ifndef _BITSET_H_
#define _BITSET_H_

#include <assert.h>
#include <string.h>
#include <stdlib.h>

///////////////////////////
//                       //
// Bare-bones bitvectors.//
//                       //
///////////////////////////

class Bitset {
protected:
  int *array;
  int _size;
  static int intLogSize; // log2 of number of bits in an int.
  static int intSize; // number of bits in an int.			   
  static inline void calcIntSize() {
    intSize = sizeof(int)*8;
    int kount = intSize;
    while (kount != 1) {
      intLogSize++;
      kount = kount >> 1;
    }
  }
  inline int getBitIndex(int index) {
    return ((array[index >> intLogSize]) >> (index % intSize) & 1);
  }
  inline void setBitIndex(int index) {
    int selectInt = index >> intLogSize;
    array[selectInt] = array[selectInt] | 
                                     ( 1 << (index % intSize));
  }
  inline void clearBitIndex(int index) {
    int selectInt = index >> intLogSize;
    array[selectInt] = array[selectInt] & 
                                     (~( 1 << (index % intSize)));
  }
public:
  Bitset(int size) : _size(size) { 
    // Silly way to do this.
    if (!intLogSize) {
      calcIntSize();
    }
    array = (int *)malloc(((size >> intLogSize) + 1) * sizeof(int));
    memset(array, 0, ((size >> intLogSize) + 1) * sizeof(int));
  }
  ~Bitset() { free(array); }
  void un(Bitset *b);
  void subtract(Bitset *b);
  bool equal(Bitset *b);
  bool test(unsigned int index) { 
    assert(index < _size); 
    return getBitIndex(index);
  }
  void set(unsigned int index) { 
    assert(index < _size); 
    setBitIndex(index);
  }
  void clear(unsigned int index) {
    assert(index < _size);
    clearBitIndex(index);
  }
  int size(void) { return _size; }
  void print(void);
};

#endif
  
