#ifndef _CFG_H_
#define _CFG_H_

#include "bitset.h"
#include "llist.h"

class Bblock;

// Class declaration
class CfgNode {
protected:
    TreeNode *ast_node;

    llist<CfgNode *> *_succ;

    llist<CfgNode *> *_pred;

    int visited;

public:
    CfgNode (TreeNode *node) {
	_succ = NULL;
	_pred = NULL;
	numSucc = numPred = 0;
	visited = 0;
	ast_node = node;
	bblock = NULL;
    }
    TreeNode *astNode() { return ast_node; }
    int numSucc;
    int numPred;
    Bblock *bblock;
    void nodeStatus(int v) { visited = v; }
    int nodeStatus() { return visited; }
    void addSucc(CfgNode *n) { numSucc++; _succ = cons(n, _succ); }
    void addPred(CfgNode *n) { numPred++; _pred = cons(n, _pred); }
    ListIterator<CfgNode *> succIter() {
	return ListIterator<CfgNode *> (_succ);
    }
    ListIterator<CfgNode *> predIter() {
	return ListIterator<CfgNode *> (_pred);
    }
};

class Bblock {
protected:
  llist<Bblock *> *_preds;
  llist<Bblock *> *_succs;
  llist<CfgNode *> *_cfglist;
  int visited;
  Bitset *_defsOut;  // defs that reach out of the bblock.
  Bitset *_defsIn;   // union of incoming defs
  Bitset *_defs;     // defs in this bblock
  Bitset *_kills;    // defs killed by this bblock
  
public:
  Bblock();
  ListIterator<Bblock *> succIter() {
    return ListIterator<Bblock *>(_succs);
  }
  ListIterator<Bblock *>predIter() {
    return ListIterator<Bblock *>(_preds);
  }
  ListIterator<CfgNode *>cfgIter() {
    return ListIterator<CfgNode *>(_cfglist);
  }
  void nodeStatus(int v) { visited = v; }
  int nodeStatus() { return visited; }
  void addSucc(Bblock *b) { _succs = cons(b, _succs); }
  void addPred(Bblock *b) { _preds = cons(b, _preds); }
  void addCfg(CfgNode *c) { 
    _cfglist = extend(_cfglist, cons(c));
    c->bblock = this; 
  }
  Bitset *defs() { return _defs; }
  Bitset *kills() { return _kills; }
  Bitset *defsIn() { return _defsIn; }
  Bitset *defsOut() { return _defsOut; }
  void setDefs(Bitset *defs) { _defs = defs; }
  void setKills(Bitset *kills) { _kills = kills; }
  void setDefsIn(Bitset *defs) { _defsIn = defs; }
  void setDefsOut(Bitset *defs) { _defsOut = defs; }
  void print(ostream &os);
  int number;
  int defsize;
};

void TraverseCfg(CfgNode *c, void action(CfgNode *));
void TraverseCfgExtents(CfgNode *from, CfgNode *to, void action(CfgNode *));
void UnmarkCfg(CfgNode *c);
void UnmarkCfgExts(CfgNode *from, CfgNode *to);
void UnmarkBblocks(Bblock *b);
void traverseCfgBblock(CfgNode *c, Bblock *currBblock, CfgNode *lastCfg);
void TraverseBblocksIter(Bblock *b, bool action(Bblock *b));
void TraverseBblocks(Bblock *b, void action(Bblock *b));
CfgNode *LastMethodNode(TreeNode *p);

#endif
