#include <cstdio>
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <assert.h>
#include <strstream.h>
#include "utils.h"
#include "AST.h"
#include "decls.h"
#include "errors.h"
#include "interface.h"
#include "code.h"
#include "config.h"
#include "code-grid.h"
#include "code-util.h"
#include "code-commonsub.h"
#include <map>

typedef map< string, llist<TreeNode *> *, less< string > > map_string_to_list;
typedef map< string, map_string_to_list, less< string > >
    map_string_string_to_list;
typedef map< string, void *, less< string > > map_string_to_void;
typedef map< TreeNode *, map_string_to_void, less< TreeNode * > >
    map_tree_string_to_void;

extern bool debug_opt;

extern bool apparentlyIdenticalChildren(const TreeNode *u, const TreeNode *t);

/* If this isn't an ExprNode or subclass of ExprNode, then recursively
   check the children but, for simplicity, always return this. */
void *TreeNode::commonSubExpr()
{
  foriter (p, allChildren(), ChildIter)
    (*p)->commonSubExpr();

  return (void *) this;
}

bool compareLists(const llist<TreeNode *> *t, const llist<TreeNode *> *u)
{
  return ((t == NULL) == (u == NULL))
    && (t == NULL
	|| COMPARE(t->front(), u->front())
	&& compareLists(t->tail(), u->tail()));
}

/* If l contains a node that compares equal to t, return it.  If none
   exists, add t to l and return t.  */
static void *search(llist<TreeNode *> *& l, TreeNode *t, bool& found)
{
  if (l == NULL) {
    l = cons(t, l);
    found = false;
    return (void *) t;
  } else if (COMPARE(l->front(), t)) {
    found = true;
    return (void *) l->front();
  } else
    return search(l->tail(), t, found);
}

static void *commonSubExpr(TreeNode *t, string s, bool childrenOnly)
{
  static map_tree_string_to_void memo[2];
  static map_string_to_list memo_search;
  static map_string_to_void memo_noSearch;

  void *& outermemo = memo[childrenOnly ? 1 : 0][t][s];
  if (outermemo != NULL) return outermemo;

  s = s + t->oper_name();
  foriter (p, t->allChildren(), TreeNode::ChildIter) {
    s = s + ' ' + long2hexstr((long) (*p)->commonSubExpr());
  }
  bool found;
  void *ret =
    childrenOnly ? memo_noSearch[s] : search(memo_search[s], t, found);
  if (childrenOnly && !((found = (ret != NULL)))) 
    ret = memo_noSearch[s] = (void *) t;

  if (debug_commonsub) {
    if (found) printf("commonSubExpr() found memo[%s] = %x\n",
		      s.c_str(), ret);
    else printf("commonSubExpr() didn't find memo[%s]\n",
		s.c_str());
  }

  return (outermemo = ret);
}

void *ExprNode::commonSubExpr()
{
  return ::commonSubExpr(this, "", false);
}

void *ArrayAccessNode::commonSubExpr()
{
  return ::commonSubExpr(this, "", true);
}

void *TreeListNode::commonSubExpr()
{
  return ::commonSubExpr(this, "", true);
}

void *NameNode::commonSubExpr()
{
  return ::commonSubExpr(this, *ident() + ' ' + long2hexstr((long) decl()),
			 true);
  // The above is equivalent but faster because it doesn't rely on searching.
  // return ::commonSubExpr(this, "", false);
}

void *StringLitNode::commonSubExpr()
{
  ostrstream s;
  printEscaped(s, text());
  s << ends;
  return ::commonSubExpr(this, s.str(), true);
}

/* FieldAccessNodes */

/* IncrDecrNodes */

/* UnaryArithNodes */

/* BinaryArithNodes */

/* BinaryArithAssignNodes */

/* ShiftNodes */

/* ShiftAssignNodes */

/* RelationNodes */

/* EqualityNodes */

/* BitwiseNodes */

/* BitwiseAssignNodes */

/* LogCondNodes */

/* DynamicTypeNodes */

// ArrayInitNode

// NullPntrNode

// ThisNode

// ArrayAccessNode

// ObjectNode

// MethodCallNode

// MethodAssignCallNode

// AllocateNode

// AllocateArrayNode

// ComplementNode

// NotNode

// StringConcatNode

// IfExprNode

// AssignNode

// PointNode

// DomainNode

