/* $Id: common.h 1.13 Fri, 30 Apr 1999 14:44:24 -0700 liblit $	 -*-C++-*-							*/
/* common.h: Assorted Definitions					*/

/* Copyright (C) 1996, Paul N. Hilfinger.  All Rights Reserved.		*/


#ifndef _COMMON_H_
#define _COMMON_H_

#include <iosfwd>
#include <string>


class Common {
public:
    /* Access and other properties of classes, fields, and methods. A */
    /* logical OR of these values represents a set of properties. */
    /* Example: the value  */
    /*   (Common::Modifiers) (Common::Public | Common::Volatile) */
    /* indicates a volatile public field. */
    enum Modifiers {
	Public = 1 << 0,
	Private = 1 << 1,
	Protected = 1 << 2,
	Static = 1 << 3,
	Final = 1 << 4,
	Synchronized = 1 << 5,
	Volatile = 1 << 6,
	Transient = 1 << 7,
	Native = 1 << 8,
	Interface = 1 << 9,
	Abstract = 1 << 10,
	Single = 1 << 11,
	Local = 1 << 12,
	LocalInferred = 1 << 13,
	Immutable = 1 << 14,
	Sglobal = 1 << 15,
	Inline = 1 << 16
    };

    /* A Kind is a category of type. */
    enum Kind {
	BoolKind = 1,
	CharKind = 2,
	ByteKind = 4,
	ShortKind = 8,
	IntKind = 16,
	LongKind = 32,
	FloatKind = 64,
	DoubleKind = 128,
	NullKind = 256,		/* The 'type' of null. */
	ClassKind = 512,
	InterfaceKind = 1024,
	MethodKind = 2048,
	VoidKind = 4096,
	ArrayKind = 8192,
	ArrayInitializerKind = 16384, /* The 'type' of array initializers. */ 
	ImmutableKind = 32768, /* Immutable classes */
	};

};

void indent(ostream &os, unsigned depth);
string stringifyModifiers (Common::Modifiers flags);

#endif
