#ifndef _LITERAL_H_
#define _LITERAL_H_

#include <string>
#include "common.h"

// Java constants, and operations for constant folding

typedef long long int64; // a 64 bit type, substitute your favourite
typedef long int32; // a 32 bit type, substitute your favourite
typedef short int16; // a 16 bit type, substitute your favourite
typedef unsigned short uint16; // a java character, i.e. 16 bits unsigned
typedef signed char int8;

class Literal {
public:
  Literal() { _kind = (Common::Kind) 0; }
  Literal(uint16 x) { _kind = Common::CharKind; i = x; }
  Literal(int32 x) { _kind = Common::IntKind; i = x; }
  Literal(int64 x) { _kind = Common::LongKind; i = x; }
  Literal(float x) { _kind = Common::FloatKind; f = x; }
  Literal(double x) { _kind = Common::DoubleKind; d = x; }
  Literal(bool x) { _kind = Common::BoolKind; b = x; }

  Common::Kind kind() const { return _kind; }
  Literal cast(Common::Kind to);

  Literal operator~();
  Literal operator!();
  Literal operator-();
  Literal operator*(Literal arg);
  Literal operator/(Literal arg);
  Literal operator%(Literal arg);
  Literal operator+(Literal arg);
  Literal operator-(Literal arg);
  Literal lsl(Literal arg); // logical shift left
  Literal rsl(Literal arg); // logical shift right
  Literal rsa(Literal arg); // arighmetic shift right
  // the damn g++ compiler feels like instantiating function.h on
  // Literal, and gets silly errors for <, <=, >, >= (in fold.cc)
  Literal lt(Literal arg);
  Literal le(Literal arg);
  Literal gt(Literal arg);
  Literal ge(Literal arg);
  Literal eq(Literal arg);
  Literal ne(Literal arg);
  Literal operator&(Literal arg);
  Literal operator|(Literal arg);
  Literal operator^(Literal arg);
  Literal operator&&(Literal arg);
  Literal operator||(Literal arg);

  // operators to extract info on the types of nodes
  bool boolValue() const { return b; }
  int64 intValue() const;
  double doubleValue() const { return kind() == Common::FloatKind ? f : d; }

  // Return a printed representation of this literal
  const string asString() const;

protected:
  Literal arithPromote();
  Literal arithPromote(Common::Kind to);
  Common::Kind _kind;
  union {
    int64 i;
    float f;
    double d;
    bool b;
  };
};

ostream& operator<<(ostream& os, Literal l);

// make literal from java token
Literal intLiteral(const string &);
Literal longLiteral(const string &);
Literal charLiteral(const string &);
Literal floatLiteral(const string &);
Literal doubleLiteral(const string &);

#define IntegerKind Common::ByteKind: case Common::ShortKind: case Common::CharKind: case Common::IntKind: case Common::LongKind

#endif
