#include "AST.h"
#include "context.h"
#include "decls.h"
#include "solver.h"


void TreeListNode::localConstrainCall( const TypeListNode &formal ) const
{
  assert( arity() == formal.arity() );
  
  for (unsigned sweep = formal.arity(); sweep--; )
    child( sweep )->type()->localWeaklySubsumedBy( *formal.child( sweep ) );
}


void TreeNode::localConstrainCall( const TypeNode &callee ) const
{
  args()->localConstrainCall( *callee.paramTypes() );
}


////////////////////////////////////////////////////////////////////////


void MethodCallAssignNode::inferLocal( const LocalContext & )
{
  const TypeNode &dispatcher = *method()->accessedObjectType();
  if (dispatcher.localInferable())
    solver().subsumedBy( dispatcher, *method()->decl()->type() );
    
  const TypeNode &target = *method()->type();
  localConstrainCall( target );
  type()->localIdenticalTo( *target.returnType() );
}


void MethodCallNode::inferLocal( const LocalContext &context )
{
  const TypeNode &dispatcher = *method()->accessedObjectType();
  if (dispatcher.localInferable())
    solver().subsumedBy( dispatcher, *method()->decl()->type() );

  const TypeNode &target = *method()->type();
  type()->localIdenticalTo( *target.returnType() );

  if (dispatcher.isTitaniumArrayType())
    {
      const TypeNode &element = *dispatcher.elementType();
      if (element.localInferable())
	{
	  static const string * const exchange = intern( "exchange" );
	  static const string * const copy = intern( "copy" );
	  const string * const name = decl()->name();
	  
	  if (name == exchange)
	    {
	      assert( args()->arity() == 1 );
	      const TypeNode &donation = *args()->child( 0 )->type();

	      solver().mustBeGlobal( element );
	      donation.localWeaklySubsumedBy( element );
	      
	      context.stale.insert( this );
	      return;
	    }

	  else if (name == copy)
	    {
	      assert( args()->arity() == 1 );
	      const TypeNode &donor = *args()->child( 0 )->type();
	      
	      solver().subsumedBy( dispatcher, element );
	      donor.localWeaklySubsumedBy( dispatcher );

	      context.stale.insert( this );
	      return;
	    }
	}
    }

  localConstrainCall( target );
}


void SuperConstructorCallNode::inferLocal( const LocalContext & )
{
  localConstrainCall( *decl()->type() );
}


void ThisConstructorCallNode::inferLocal( const LocalContext & )
{
  localConstrainCall( *decl()->type() );
}
