/* -*-C++-*-							*/
/* parse.h: Communication interface between the parser, the lexer, */
/* and the world. */

#ifndef _PARSER_H_
#define _PARSER_H_

#include <cstddef>
#include <string>
#include <vector>
#include "code.h"

class TreeNode;
class ostream;


class File
{
public:
  File(const string *__name) : _name(__name), lineEndings(1, 0u) { }
  const string *name() { return _name; }

  /* The line number on which character N of the input apppears. */
  size_t posnToLineNumber (unsigned long N);

  /* Record that N is the position of a newline in the source program */
  /* file.  It is assumed that this function is called with increasing */
  /* values of N. */
  void defineLineEnd (unsigned long N);
private:
  const string *_name;
  vector<unsigned long> lineEndings;
};

/* A SourcePosn is a character number (starting at 1) within a file. */
/* The position of a token is identified by the SourcePosn of its */
/* first character.  */
struct SourcePosn
{
  bool unknown() const { return file == NULL; }

  /* The line number on which character N of the input apppears. */
  size_t posnToLineNumber () const {
    return unknown() ? 0 : file->posnToLineNumber(posn);
  }

  const string asString() const {
    return unknown() ?
      string("unknown source position")
      : (*(file->name()) +
	 ':' +
	 (const char *) int2str((int)posnToLineNumber()));
  }

  void dump( ostream & ) const;

  File *file;
  unsigned long posn;

};

/* The undefined position.  Used to as the position of nodes that */
/* have no meaningful position in the source. */
extern const SourcePosn NoSourcePosition;
    
/* Print a position in a format suitable for errors or warnings. */
ostream & operator << ( ostream &, const SourcePosn & );

/* The parser's interface to the rest of the compiler.  The */
/* root of the synthesized abstract syntax tree is to be passed to */
/* this function. */
void compileAST(TreeNode* root);

/* The unique string* containing the current contents of TEXT. */
const string* intern(const char* text);
const string* intern(const string &);

void parse(string name, bool titaniumFile);

#endif
