#ifndef _INCLUDE_MEM_WIDE_H_
#define _INCLUDE_MEM_WIDE_H_

#define WIDE_POINTERS

#include "instance_header.h"
#include "glob_ptr_base.h"
#include "mem-local.h"


/* PLASSIGN_GLOBAL is illegal. */

#define PGASSIGN_GLOBAL(ptr, val) \
  do { if (!(!isLocal(val) || SHARED_REGIONID(PL2RegionId(TO_LOCAL(val)))) && \
	   (!isLocal(ptr) || SHARED_REGIONID(PL2RegionId(TO_LOCAL(ptr))))) \
	   abort(); \
	else ASSIGN_GLOBAL(ptr, val); } while (0)

#define SPLASSIGN_GLOBAL	ASSIGN_GLOBAL
#define SPGASSIGN_GLOBAL	ASSIGN_GLOBAL

/* WEAK_PLASSIGN_GLOBAL is illegal. */

#define WEAK_PGASSIGN_GLOBAL(ptr, val) \
  do { if (!(!isLocal(val) || SHARED_REGIONID(PL2RegionId(TO_LOCAL(val)))) && \
	   (!isLocal(ptr) || SHARED_REGIONID(PL2RegionId(TO_LOCAL(ptr))))) \
	   abort(); \
	else WEAK_ASSIGN_GLOBAL(ptr, val); } while (0)

#define WEAK_SPLASSIGN_GLOBAL	WEAK_ASSIGN_GLOBAL	
#define WEAK_SPGASSIGN_GLOBAL	WEAK_ASSIGN_GLOBAL	

#define FIELD_ADDR_GLOBAL(val, obj, fld)	TO_GLOBAL(val, obj.proc, &obj.addr->fld)
#define IFACE_DISPATCH_GLOBAL(p, mid)		(IFACE_DISPATCH_LOCAL(TO_LOCAL(p), mid))
#define INDEX_GLOBAL(element, base, offset)	TO_GLOBAL(element, base.proc, base.addr + offset)
#define EQUAL_GLOBAL(p, q)			((isNull(p) && isNull(q)) || ((TO_PROC(p) == TO_PROC(q)) && (TO_LOCAL(p) == TO_LOCAL(q))))

#define CLASS_INFO_GLOBAL( val, type, obj )		\
  do {							\
    GP_LP_ ## type field_;				\
    FIELD_ADDR_GLOBAL( field_, obj, class_info );	\
    DEREF_GLOBAL( val, field_ );			\
  } while (0)

#define _PRE_GLOBAL( val, ptr, op )		\
  do {						\
    DEREF_GLOBAL( val, ptr );			\
    op ## op ## val;				\
    ASSIGN_GLOBAL( ptr, val );			\
  } while (0)

#define PRE_INCR_GLOBAL( val, ptr )  _PRE_GLOBAL( val, ptr, + );
#define PRE_DECR_GLOBAL( val, ptr )  _PRE_GLOBAL( val, ptr, - );


#define _POST_GLOBAL( val, ptr, op )		\
  do {						\
    DEREF_GLOBAL( val, ptr );			\
    ASSIGN_GLOBAL( ptr, val op 1 );		\
  } while (0)

#define POST_INCR_GLOBAL( val, ptr )  _POST_GLOBAL( val, ptr, + );
#define POST_DECR_GLOBAL( val, ptr )  _POST_GLOBAL( val, ptr, - );


#define TO_LOCAL(ptr)  ((ptr).addr)
#define TO_PROC(ptr)   ((ptr).proc)

#define TO_GLOBAL( global, processor, address )	\
  do {						\
    (global).proc = (processor);		\
    (global).addr = (address);			\
  } while (0)


#define isLocal( global )	(TO_PROC(global) == MYPROC || isNull(global))

#define globalize( global, local )	TO_GLOBAL( global, MYPROC, local )

#define localize( local, global )				\
  do {								\
    if (isLocal(global))					\
      (local) = TO_LOCAL( global );				\
    else							\
      cl_ti_lang_NativeUtils_mt_throwClassCastException();	\
  } while (0)


#endif /* !_INCLUDE_MEM_WIDE_H_ */
